unit uPIPTree;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls;

type
  TPIPTreeView = class(TCustomTreeView)
  private
    { Private declarations }
    fTreeSList: TStringList;
  protected
    { Protected declarations }
    procedure Expand(Node: TTreeNode); override;
    procedure Collapse(Node: TTreeNode); override;
  public
    { Public declarations }
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure RestoreExpand;
     procedure SaveExpand(ts: TStream);
     procedure LoadExpand(ts: TStream);


  published
    { Published declarations }
    property ShowButtons;
    property BorderStyle;
    property DragCursor;
    property ShowLines;
    property ShowRoot;
    property ReadOnly;
    property RightClickSelect;
    property DragMode;
    property HideSelection;
    property Indent;
    property Items;
    property OnEditing;
    property OnEdited;
    property OnExpanding;
    property OnExpanded;
    property OnCollapsing;
    property OnCompare;
    property OnCollapsed;
    property OnChanging;
    property OnChange;
    property OnDeletion;
    property OnGetImageIndex;
    property OnGetSelectedIndex;
    property Align;
    property Enabled;
    property Font;
    property Color;
    property ParentColor default False;
    property ParentCtl3D;
    property Ctl3D;
    property SortType;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDrag;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property PopupMenu;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Images;
    property StateImages;
  end;

procedure Register;

implementation

{TPIPTreeView}
{============================================================================}
constructor TPIPTreeView.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 fTreeSList:=TStringList.Create;
 fTreeSList.Clear;
end;

{============================================================================}
destructor TPIPTreeView.Destroy;
begin
 fTreeSList.Free;
 inherited Destroy;
end;

{============================================================================}
procedure TPIPTreeView.Expand(Node: TTreeNode);
var
i: longint;
begin
    if fTreeSList.IndexOf(Node.Text) < 0 then
    begin
         fTreeSList.Add(Node.Text);
    end;
    inherited Expand(Node);
end;

{============================================================================}
procedure TPIPTreeView.Collapse(Node: TTreeNode);
var
i: longint;
begin
 i:=fTreeSList.IndexOf(Node.Text);
 if i >= 0 then
    fTreeSList.Delete(i);
 inherited Collapse(Node);
end;

{============================================================================}
procedure TPIPTreeView.RestoreExpand;
var
i,ii: longint;
cNode: TTreeNode;
tSL: TStringList;
begin
try
     cNode:=Items.GetFirstNode;
     tSL:=TStringList.Create;
     while cNode <> nil do
     begin
          ii:=fTreeSList.IndexOf(cNode.Text);
          if ii >= 0 then
             cNode.Expand(False)
          else
             tSL.Add(cNode.Text);
          cNode:=cNode.GetNext;
     end;
     for i:=0 to tSL.Count-1 do
     begin
          ii:=fTreeSList.IndexOf(tSL.Strings[i]);
          if ii >= 0 then
             fTreeSList.Delete(ii);
     end;
finally
    tSL.Free;
end;

end;

{============================================================================}
procedure TPIPTreeView.SaveExpand(ts: TStream);
begin
     fTreeSList.SaveToStream(ts);
end;

{============================================================================}
procedure TPIPTreeView.LoadExpand(ts: TStream);
begin
     fTreeSList.Clear;
     fTreeSList.LoadFromStream(ts);
end;

{============================================================================}
procedure Register;
begin
  RegisterComponents('Misc', [TPIPTreeView]);
end;

end.
